#include <assert.h>
#include <stdbool.h>
#include <stdio.h>
#include <stdlib.h>
#include "array_helpers.h"


static bool file_exists(const char *filepath) {
    bool result = false;
    FILE *file = NULL;

    assert(filepath != NULL);

    file = fopen(filepath, "r");

    result = (file != NULL);
    if (result) {
        fclose(file);
    }

    return(result);
}

bool array_is_sorted(int *a, unsigned int length) {
    bool result = true;
    unsigned int i = 0;

    assert(a != NULL);

    for (i = 0; i < (length - 1); i++) {
        result = result && (a[i] <= a[i+1]);
    }

    return(result);
}

bool array_is_equal(int *a, int *other, unsigned int length) {
    bool result = true;
    unsigned int i = 0;

    assert(a != NULL);
    assert(other != NULL);

    for (i = 0; i < length; i++) {
        result = result && (a[i] == other[i]);
    }

    return(result);
}

bool array_has_value(int *a, unsigned int length, int value) {
    bool result = false;
    unsigned int i = 0;

    assert(a != NULL);

    for (i = 0; i < length; i++) {
        result = result || (a[i] == value);
    }

    return(result);
}

unsigned int array_value_count(int *a, unsigned int length, int value) {
    unsigned int i = 0, result = 0;

    assert(a != NULL);

    for (i = 0; i < length; i++) {
        if (a[i] == value) {
            result += 1;
        }
    }

    return(result);
}

bool array_is_permutation_of(int *a, int *other, unsigned int length) {
    bool result = true;
    unsigned int i = 0, a_count = 0, other_count = 0;

    assert(a != NULL);
    assert(other != NULL);

    for (i = 0; i < length; i++) {
        /* a[i] is also present in 'other' */
        result = result && array_has_value(other, length, a[i]);

        /* courrences of a[i] in 'a' match those in 'other' */
        a_count = array_value_count(a, length, a[i]);
        other_count = array_value_count(other, length, a[i]);
        result = result && (a_count == other_count);
    }

    return(result);
}

int *array_create(unsigned int length) {
    int *result = NULL;

    result = calloc(length, sizeof(int));
    assert(result != NULL);

    return(result);
}

void array_destroy(int *a) {
    free(a);
}

void array_dump(int *a, unsigned int length) {
    unsigned int i = 0;

    assert(a != NULL);

    fprintf(stderr, "%u\n", length);
    for (i = 0; i < length; i++) {
        fprintf(stderr, "%i", a[i]);
        if (i < length - 1) {
            fprintf(stderr, " ");
        } else {
            fprintf(stderr, "\n");
        }
    }
}

int *array_duplicate(int *a, unsigned int length) {
    unsigned int i = 0;
    int *result = NULL;

    assert(a != NULL);

    result = array_create(length);
    for (i = 0; i < length; i++) {
        result[i] = a[i];
    }

    assert(result != NULL);
    assert(array_is_equal(a, result, length));

    return(result);
}

unsigned int array_length_from_file(const char *filepath) {
    int fscanf_result = 0;
    unsigned int result = 0;
    FILE *file = NULL;

    /* Check preconditions */
    assert(file_exists(filepath));

    file = fopen(filepath, "r");
    assert(file != NULL);

    fscanf_result = fscanf(file, "%u\n", &result);

    fclose(file);

    if (fscanf_result != 1) {
        printf("The file does not have the correct format for the array "
               "length.\n");
        exit(EXIT_FAILURE);
    }

    return(result);
}

int *array_from_file(const char *filepath, unsigned int length) {
    int fscanf_result = 0;
    unsigned int i = 0, other_length = 0;
    int *result = NULL;
    FILE *file = NULL;

    /* Check preconditions */
    assert(file_exists(filepath));

    result = array_create(length);

    file = fopen(filepath, "r");
    assert(file != NULL);

    fscanf_result = fscanf(file, "%u\n", &other_length);
    if (fscanf_result != 1) {
        printf("The file does not have the correct format for the array "
               "length.\n");
        exit(EXIT_FAILURE);
    }

    if (length != other_length) {
        printf("The array length in the file (%u) does not match the given "
               "length (%u).\n", other_length, length);
        exit(EXIT_FAILURE);
    }

    for (i = 0; i < length; i++) {
        fscanf_result = fscanf(file, "%i", &(result[i]));
        if (fscanf_result != 1) {
            printf("The file does not have the correct format for the array "
                   "elements.\n");
            exit(EXIT_FAILURE);
        }
    }

    fclose(file);

    assert(result != NULL);

    return(result);
}
